import pathlib
from pathlib import Path
from django.contrib import admin
from django.contrib.admin import AdminSite
from django.conf import settings
from adminvisualsortable.views import get_fk_name, get_model_inlines
from django.db.models import ForeignKey

class AdminVisualSortableInlineMixin:
    ''' Enable sortable for Inline Models '''
    is_adminvisualsortable = True

class AdminVisualSortableBaseMixin:
    ''' Admin Model mandatory if Inlines are Sortables'''
    is_adminvisualsortable = False
    @property
    def change_form_template(self):
        if getattr(self, 'add_tabbed_item', None):
            return str(Path('tabbed_change_form.html'))
        else:
            return str(Path('change_form.html'))

    def has_sort_elements(self, object_id):
        ''' Check if Parent Model has Sortable Inlines '''
        TargetModel = self.model.__name__
        inline_list = get_model_inlines(TargetModel)
        for ModelInline in inline_list:
            fields = ModelInline._meta.get_fields()
            for f in fields:
                if f.related_model == self.model:
                    if ModelInline.objects.filter(**{f.name:object_id}).count() > 0:
                        return True
            if ModelInline.objects.filter(id=object_id).count() > 0:
                return True
        return False

    def change_view(self, request, object_id, form_url='', extra_context=None):
        extra_context = extra_context or {}
        extra_context['is_adminvisualsortable'] = self.is_adminvisualsortable
        extra_context['has_sort_elements'] = self.has_sort_elements(object_id)
        return super().change_view(
            request, object_id, form_url, extra_context=extra_context,
        )

class AdminVisualSortableMixin(AdminVisualSortableBaseMixin):
    ''' Admin Model Mixin for Sortable Model'''
    is_adminvisualsortable = True
    @property
    def change_list_template(self):
        opts = self.model._meta
        app_label = opts.app_label
        return str(Path('change_list.html'))



#-*- coding: utf-8 -*-
from django.urls import path, include, re_path
from adminvisualsortable import views

app_name = 'adminvisualsortable'

urlpatterns_base = [
    path('<slug:app>/<slug:modelname>/', views.sort_models, name='sort models'), # Ordina i Models
    
    path('<slug:app>/<slug:modelparentname>/<int:model_id>/sortinlines/', views.sortable_inlines, name='sortable inlines'), # Lista inlines ordinabili
    path('<slug:app>/<slug:modelparentname>/<int:model_id>/sortinlines/<slug:modelname>/<slug:fkfield>/', views.sortable_inlines_detail, name='sortable inlines detail'), # Ordina gli Inlines
    path('<slug:app>/<slug:modelparentname>/<int:model_id>/sortinlines/<slug:modelname>/', views.sortable_inlines_detail, name='sortable inlines detail'), # Ordina gli Inlines


    
]

urlpatterns_api = [
    path('api/sort-model/', views.api_sort_models, name='api sort models'), # salva le modifiche ordinamento Models
]

urlpatterns = urlpatterns_api + urlpatterns_base



#-*- coding: utf-8 -*-
from django import template
from django.urls import reverse
from django.conf import settings

register = template.Library()

@register.filter(name='get_child_sortable_list')
def get_child_sortable_list(value):
    try:
        value_list = value.split('/')
        obj_id = value_list[-3]
        model_name = value_list[-4]
        app_name = value_list[-5]
    except ValueError:
        return None
    except IndexError:
        return None
    return reverse('adminvisualsortable:sortable inlines', kwargs={'app': app_name, 'modelparentname': model_name, 'model_id': obj_id })


@register.filter(name='get_parent_visualorder_url')
def get_parent_visualorder_url(value):
    obj = value
    return '/adminvisualsortable/{}/{}/'.format(obj._meta.app_label, obj.__class__.__name__.lower())


@register.filter(name='get_parent_admin_url_detail')
def get_parent_admin_url_detail(value):
    obj = value
    return '/admin/{}/{}/{}'.format(obj._meta.app_label, obj.__class__.__name__.lower(), obj.id)


@register.filter(name='get_parent_admin_url_list')
def get_parent_admin_url_list(value):
    model = value
    modelname = model['name'].lower()
    app = model['app_name']
    return '/admin/{}/{}/'.format(app, modelname)



#!/usr/bin/env python

if __name__ == "__main__":
    import setuptools
    setuptools.setup(
        package_data={
            '': [ 'templates/*.html', 
                  'templates/adminvisualsortable/*.html', 
                  'static/adminvisualsortable/css/yui/*.css',
                  'static/adminvisualsortable/css/*.css',
                  'static/adminvisualsortable/js/*.js',
                  'static/adminvisualsortable/js/*.map',
                  'static/adminvisualsortable/img/*.png'
                ]
            },
    )
